.inline-block {
    display: inline;
    vertical-align: baseline;
    margin-bottom:-2em;
    margin-top:2em;
    position:relative;
    top:-2em;
}
.vertical-align-middle-middlebox {
    display: block;
    height: auto;
    position: absolute;
    top: 50%;
}
.vertical-align-middle-innerbox {
    display: block;
    position: relative;
    top: -50%;
}
li.full-width {
    width: auto;
}
li div div.inline-block a img {
    text-indent: 0;
}
img {
    -ms-interpolation-mode: bicubic;
}
