// Created by iWeb 3.0.4 local-build-20120926

setTransparentGifURL('Media/transparent.gif');function applyEffects()
{var registry=IWCreateEffectRegistry();registry.registerEffects({shadow_0:new IWShadow({blurRadius:6,offset:new IWPoint(2.8284,2.8284),color:'#000000',opacity:0.500000}),reflection_0:new IWReflection({opacity:0.50,offset:1.00}),stroke_0:new IWStrokeParts([{rect:new IWRect(-1,1,2,212),url:'About_Us_files/stroke.png'},{rect:new IWRect(-1,-1,2,2),url:'About_Us_files/stroke_1.png'},{rect:new IWRect(1,-1,317,2),url:'About_Us_files/stroke_2.png'},{rect:new IWRect(318,-1,3,2),url:'About_Us_files/stroke_3.png'},{rect:new IWRect(318,1,3,212),url:'About_Us_files/stroke_4.png'},{rect:new IWRect(318,213,3,2),url:'About_Us_files/stroke_5.png'},{rect:new IWRect(1,213,317,2),url:'About_Us_files/stroke_6.png'},{rect:new IWRect(-1,213,2,2),url:'About_Us_files/stroke_7.png'}],new IWSize(320,214)),shadow_1:new IWShadow({blurRadius:4,offset:new IWPoint(1.4142,1.4142),color:'#000000',opacity:0.500000})});registry.applyEffects();}
function hostedOnDM()
{return false;}
function onPageLoad()
{loadMozillaCSS('About_Us_files/About_UsMoz.css')
adjustLineHeightIfTooBig('id1');adjustFontSizeIfTooBig('id1');adjustLineHeightIfTooBig('id2');adjustFontSizeIfTooBig('id2');adjustLineHeightIfTooBig('id3');adjustFontSizeIfTooBig('id3');adjustLineHeightIfTooBig('id4');adjustFontSizeIfTooBig('id4');adjustLineHeightIfTooBig('id5');adjustFontSizeIfTooBig('id5');adjustLineHeightIfTooBig('id6');adjustFontSizeIfTooBig('id6');Widget.onload();fixAllIEPNGs('Media/transparent.gif');fixupIECSS3Opacity('id7');applyEffects()}
function onPageUnload()
{Widget.onunload();}
